@@ -139,6 +139,20 @@ log_action *act;
 }
 
 
+Static Void log_dump_32(act)
+log_action *act;
+{
+  strlist_append(&act->actstrlist, (char *)act->actproc);
+}
+
+
+Static Void log_read_32(act)
+log_action *act;
+{
+  strcpy((char *)act->actproc, act->actstrlist->s);
+}
+
+
 Static Void log_pass_32()
 {
   short Newgatenum;
@@ -865,7 +879,18 @@ log_action *act;
     break;
 
   case act_edit:
-    log_edit_32(act);
+    switch (act->acty) {
+    case 0:
+      log_edit_32(act);
+      break;
+    case 1:
+    case 2:
+      log_dump_32(act);
+      break;
+    case 3:
+      log_read_32(act);
+      break;
+    }
     break;
 
   case act_pass:   /*Iversion*/
