@@ -1778,7 +1778,7 @@ long *Totnodes;
       if (*Ok) {
 	if (*Totnodes == 0) {
 	  *Ok = false;
-	  AnaScoreboard("No nodes unconstained", (long)AnaMessScoreboard);
+	  AnaScoreboard("No nodes unconstrained", (long)AnaMessScoreboard);
 	}
 	if (*Totnodes > AnaRowbound) {
 	  *Ok = false;
