@@ -1394,6 +1394,7 @@ log_nrec **n;
 
   WITH = lact;
   (*WITH->hook.newnode)(n, log_16_simtype);
+  (*n)->keep = true;
   (*n)->ref = 1;
 }
 
@@ -1405,6 +1406,7 @@ log_nrec **n;
   log_action *WITH;
 
   WITH = lact;
+  (*n)->keep = false;
   (*WITH->hook.switchnode)(n, NULL);
 }
 
